var gl;
var useLighting;
var directionalLighting;
var pointLighting;
var specularLighting;
var cube36;


function webGLStart() {
	var canvas = document.getElementById("lumiere-canvas");
	initGL(canvas); // Initialise le context GL
	initShaders();
	initBuffers();

	gl.clearColor(0.0, 0.0, 0.0, 1.0); // Couleur : noire, opacite : 100
	gl.clearDepth(1.0);									// Clear everything
	gl.enable(gl.DEPTH_TEST);						// Enable depth testing
	gl.depthFunc(gl.LEQUAL);						// Near things hide far things
    gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);  // Clear the color as well as the depth buffer. 

	/* Mouse and keyboard interaction functions */
	document.onkeydown = tbHandleKeyDown;
    document.onkeyup = tbHandleKeyUp;
	canvas.addEventListener('mousedown', tbMouseDown, true);
	canvas.addEventListener('mousemove', tbMouseMove, true);
	canvas.addEventListener('mouseup', tbMouseUp, true);	
    
	trackball  = new TrackBall(); 
    trackball.setTranslation(0.,0.,-5.);

	drawScene();	
}


function initGL(canvas) {
	try {
		// 'experimental-webgl' est utilise depuis la phase de developpement de
		// la specification de webGL il devrait etre remplace par 'webgl' un jour
		gl = canvas.getContext("experimental-webgl");
		gl.viewportWidth = canvas.width;
		gl.viewportHeight = canvas.height;
	} catch (e) {
	}
	if (!gl) {
		alert("Could not initialise WebGL, sorry :-(");
	}
}


/*----------------------------------------------------------------------
 * Initialisation des Buffers 
 *---------------------------------------------------------------------*/
function initBuffers() {
	
	cube36 = new Cube36Normals();

		
}


/*----------------------------------------------------------------------
 * Initialisation des Shaders
 *---------------------------------------------------------------------*/
var shaderProgram;

function initShaders() {
	var fragmentShader = getShader(gl, "shader-fs");
	var vertexShader = getShader(gl, "shader-vs");

	shaderProgram = gl.createProgram();
	gl.attachShader(shaderProgram, vertexShader);
	gl.attachShader(shaderProgram, fragmentShader);
	gl.linkProgram(shaderProgram);

	if (!gl.getProgramParameter(shaderProgram, gl.LINK_STATUS)) {
		alert("Could not initialise shaders");
	}

	gl.useProgram(shaderProgram);

	/* positions */
	shaderProgram.vertexPositionAttribute = gl.getAttribLocation(shaderProgram, "aVertexPosition");
	gl.enableVertexAttribArray(shaderProgram.vertexPositionAttribute);

	/* normales */
	shaderProgram.vertexNormalAttribute = gl.getAttribLocation(shaderProgram, "aVertexNormal");
	/* NORMALES */
	gl.enableVertexAttribArray(shaderProgram.vertexNormalAttribute);  
	

	/* couleurs */
	shaderProgram.vertexColorAttribute = gl.getAttribLocation(shaderProgram, "aVertexColor");
	gl.enableVertexAttribArray(shaderProgram.vertexColorAttribute);

	shaderProgram.pMatrixUniform = gl.getUniformLocation(shaderProgram, "uPMatrix");
	shaderProgram.mvMatrixUniform = gl.getUniformLocation(shaderProgram, "uMVMatrix");
	shaderProgram.nMatrixUniform = gl.getUniformLocation(shaderProgram, "uNMatrix");
	shaderProgram.useLightingUniform = gl.getUniformLocation(shaderProgram, "uUseLighting");
	shaderProgram.usePointLightingUniform = gl.getUniformLocation(shaderProgram, "uUsePointLighting");
	shaderProgram.useDirectionalLightingUniform = gl.getUniformLocation(shaderProgram, "uUseDirectionalLighting");
	shaderProgram.useSpecularLightingUniform = gl.getUniformLocation(shaderProgram, "uUseSpecularLighting");
	shaderProgram.materialShininessUniform = gl.getUniformLocation(shaderProgram, "uMaterialShininess");
	shaderProgram.ambientColorUniform = gl.getUniformLocation(shaderProgram, "uAmbientColor");
	shaderProgram.lightingDirectionUniform = gl.getUniformLocation(shaderProgram, "uLightingDirection");
	shaderProgram.directionalColorUniform = gl.getUniformLocation(shaderProgram, "uDirectionalColor");
	shaderProgram.pointLightingLocationUniform = gl.getUniformLocation(shaderProgram, "uPointLightingLocation");
	shaderProgram.pointLightingDiffuseColorUniform = gl.getUniformLocation(shaderProgram, "uPointLightingDiffuseColor");
	shaderProgram.pointLightingSpecularColorUniform = gl.getUniformLocation(shaderProgram, "uPointLightingSpecularColor");
}


/*----------------------------------------------------------------------
 * Dessin de la scene 
 *---------------------------------------------------------------------*/
function drawScene() {
	gl.viewport(0, 0, gl.viewportWidth, gl.viewportHeight);
	gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
	
	/* on choisit l'angle de vue et les dimensions de la scene */
	mat4.perspective(45, gl.viewportWidth / gl.viewportHeight, 0.1, 100.0, pMatrix);

	// set viewing transformation
    trackball.setMatrix(mvMatrix);

	
	// setup shader
 	gl.useProgram(shaderProgram);
    pointSizeUniformLocation = gl.getUniformLocation(shaderProgram, "pointSize");
    gl.uniform1f(pointSizeUniformLocation, 15.0);
	setMatrixUniforms(shaderProgram);
    
    tuneLighting();

	cube36.draw( shaderProgram );

}

/*----------------------------------------------------------------------
 * Turn on/off the lights, define their colors and positions/directions
 *---------------------------------------------------------------------*/
function tuneLighting() {
	useLighting = true;
	directionalLighting = true;
	pointLighting = false;
	specularLighting = false;
	gl.uniform1i(shaderProgram.useLightingUniform, useLighting);
	gl.uniform1i(shaderProgram.useDirectionalLightingUniform, directionalLighting);
	gl.uniform1i(shaderProgram.usePointLightingUniform, 1);
	gl.uniform1i(shaderProgram.useSpecularLightingUniform, specularLighting);

	/* lumiere ambiante */
	var ambientR = 0.2;
	var ambientV = 0.2;
	var ambientB = 0.2;

	if (useLighting) {
		gl.uniform3f( shaderProgram.ambientColorUniform, ambientR, ambientV, ambientB)
	}

	/* direction d'une source de lumiere eloignee */
	var lightingX = 0.0;
	var lightingY = 1.0;
	var lightingZ = 1.0;
	if (useLighting && directionalLighting) {
		var lightingDirection = [lightingX, lightingY, lightingZ];
		var adjustedLD = vec3.create();
		vec3.normalize(lightingDirection, adjustedLD);
		gl.uniform3fv(shaderProgram.lightingDirectionUniform, adjustedLD);
        //console.log("light direction ", adjustedLD[0], adjustedLD[1], adjustedLD[2] );

		var directionalR = 1.0;
		var directionalV = 1.0;
		var directionalB = 1.0;
		gl.uniform3f(shaderProgram.directionalColorUniform,
				directionalR, directionalV, directionalB);
	}

	/* lumiere ponctuelle */
 	if (useLighting && pointLighting) {
        var pos = trackball.getTranslation();
        //console.log("trackball translation: ", pos );
		var lightPosX = 0.;
		var lightPosY = 0.;
		var lightPosZ = 0.;
		gl.uniform3f(shaderProgram.pointLightingLocationUniform, lightPosX, lightPosY, lightPosZ);

		var pointR = 0.9;
		var pointV = 0.9;
		var pointB = 0.9;
		gl.uniform3f(shaderProgram.pointLightingDiffuseColorUniform, pointR, pointV, pointB);

		/* reflets */
		if (specularLighting) {
			var materialShininess = 4.0;
			gl.uniform1f(shaderProgram.materialShininessUniform, materialShininess);

			var highlightR = 1.0;
			var highlightV = 1.0;
			var highlightB = 1.0;
			gl.uniform3f(shaderProgram.pointLightingSpecularColorUniform, highlightR, highlightV, highlightB);
		}
	}
}


/*----------------------------------------------------------------------
 * Matrix utility functions
 *---------------------------------------------------------------------*/
var mvMatrix = mat4.create();
var mvMatrixStack = [];
var pMatrix = mat4.create();

function mvPushMatrix() {
	var copy = mat4.create();
	mat4.set(mvMatrix, copy);
	mvMatrixStack.push(copy);
}

function mvPopMatrix() {
	if (mvMatrixStack.length == 0) {
		throw "Invalid popMatrix!";
	}
	mvMatrix = mvMatrixStack.pop();
}

function setMatrixUniforms() {
	gl.uniformMatrix4fv(shaderProgram.pMatrixUniform, false, pMatrix);
	gl.uniformMatrix4fv(shaderProgram.mvMatrixUniform, false, mvMatrix);

	/* NORMALES */
	var normalMatrix =mat3.create();
	mat4.toInverseMat3(mvMatrix, normalMatrix);
	mat3.transpose(normalMatrix);
	gl.uniformMatrix3fv(shaderProgram.nMatrixUniform, false, normalMatrix);

}

